#include "dxstdafx.h"
#include "Textures.hpp"

const unsigned short N = 3;

Textures::Textures(void) : _vbuffer(0), _ibuffer(0),
	yaw(0), pitch(0), zmove(3), wireframe(false)
{
	lights[0] = lights[1] = lights[2] = false;
}

Textures::~Textures(void)
{
}

HRESULT Textures::onCreateDevice()
{
	VERTEXFORMAT vs[N*N];

	for (int i=0; i<N; ++i)
		for (int j=0; j<N; ++j)
		{
			vs[N*i+j].v = D3DXVECTOR3(1.0f*j/(N-1)-0.5f, 0, 0.5f-1.0f*i/(N-1));
			vs[N*i+j].n = D3DXVECTOR3(0, 1, 0);
		}

		HRESULT hr;
		V( _device->CreateVertexBuffer(sizeof(VERTEXFORMAT) * N*N, 0, FVF,
			D3DPOOL_MANAGED, &_vbuffer, 0) );
		void * vdata;
		V( _vbuffer->Lock(0, sizeof(VERTEXFORMAT) * N*N, &vdata, 0) );
		CopyMemory(vdata, vs, sizeof(VERTEXFORMAT) * N*N);
		_vbuffer->Unlock();

		// Index buffer
		const unsigned int isSize = (N-1)*N*2;
		unsigned short is[isSize];
		unsigned short *isp = is;

		for (unsigned short i=0; i<N-1; ++i)
		{
			*isp++ = N*(i+1);
			*isp++ = N*i;
			for (unsigned short j=1; j<N; ++j)
			{
				*isp++ = N*(i+1)+j;
				*isp++ = N*i+j;
			}
		}


		V( _device->CreateIndexBuffer(sizeof(short) * isSize, 0, D3DFMT_INDEX16,
			D3DPOOL_MANAGED, &_ibuffer, 0) );
		V( _ibuffer->Lock(0, sizeof(short) * isSize, &vdata, 0) );
		CopyMemory(vdata, is, sizeof(short) * isSize);
		_ibuffer->Unlock();

		return D3D_OK;
}


HRESULT Textures::onResetDevice()
{
	D3DXMATRIX pers;

	D3DXMatrixPerspectiveFovLH(&pers, 3.1415f*45/180,
		(float)getWidth()/getHeight(), 0.1f, 10);
	_device->SetTransform(D3DTS_PROJECTION, &pers);

	setLights();

	setWorldViewMatrix();

	return D3D_OK;
}


void Textures::onDestroyDevice()
{
	SAFE_RELEASE(_vbuffer);
	SAFE_RELEASE(_ibuffer);
}


void Textures::onFrameMove( double fTime, float fElapsedTime )
{
}

void Textures::onFrameRender( double fTime, float fElapsedTime )
{
	_device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
		0xFF6495ED, 1, 0);

	if ( SUCCEEDED(_device->BeginScene()) )
	{
		_device->SetFVF(FVF);
		_device->SetStreamSource(0, _vbuffer, 0, sizeof(VERTEXFORMAT));
		_device->SetIndices(_ibuffer);

		_device->SetTransform(D3DTS_WORLD, &world);
		drawGrid();

		D3DXMATRIX rot, tra;
		D3DXMatrixRotationX(&rot, -D3DX_PI/2);
		D3DXMatrixTranslation(&tra, 0, 0.5f, 0.5f); 
		rot *= tra * world;
		_device->SetTransform(D3DTS_WORLD, &rot);
		drawGrid();


		D3DXMatrixRotationZ(&rot, -D3DX_PI/2);
		D3DXMatrixTranslation(&tra, -0.5f, 0.5f, 0); 
		rot *= tra * world;
		_device->SetTransform(D3DTS_WORLD, &rot);
		drawGrid();

		_device->EndScene();
	}
}

void Textures::keyboardProc( UINT nChar, bool bKeyDown, bool bAltDown )
{
	static bool pressed = false;
	if (bKeyDown && !pressed)
	{
		int lidx = nChar-'1';
		switch (nChar)
		{
		case '1':
		case '2':
		case '3':
			lights[lidx] = !lights[lidx];
			_device->LightEnable( lidx, lights[lidx] );
			break;
		case 'W':
			wireframe = !wireframe;
			if (wireframe)
				_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
			else
				_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
			break;
		case VK_UP:
			zmove -= 0.1f;
			setWorldViewMatrix();
			break;
		case VK_DOWN:
			zmove += 0.1f;
			setWorldViewMatrix();
			break;
		}
	}
	pressed = bKeyDown;
}

void Textures::mouseProc(bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown,
					  bool bSideButton1Down, bool bSideButton2Down, INT nMouseWheelDelta,
					  INT xPos, INT yPos )
{
	static int oldX, oldY;
	static bool pressed = false;

	bool changed = false;

	if (bLeftButtonDown)
	{
		if (!pressed)
			pressed = true;
		else {
			yaw += 2.0f*(oldX-xPos)/this->getWidth();
			pitch += 2.0f*(oldY-yPos)/this->getHeight();
			changed = true;
		}
		oldX = xPos;
		oldY = yPos;
	} else {
		if (pressed)
		{
			pressed = false;
			changed = true;
		}
	}
	if (changed)
		setWorldViewMatrix();
}

void Textures::drawGrid()
{
	for (int i=0; i<N-1; ++i)
		_device->DrawIndexedPrimitive(D3DPT_TRIANGLESTRIP, 0, i*N, 2*N, i*((N-1)*2+2), (N-1)*2);
}

void Textures::setWorldViewMatrix()
{
	D3DXMATRIX r1, r2, t;
	D3DXMatrixRotationX(&r2, pitch);
	D3DXMatrixRotationY(&r1, yaw);
	D3DXMatrixTranslation(&t, 0, 0, -0.5f);
	world = r1 * r2 * t;

	D3DXMatrixRotationX(&r1, -3.1415f/4);
	D3DXMatrixTranslation(&t, 0, 0, zmove);
	r1 *= t;
	_device->SetTransform(D3DTS_VIEW, &r1);
}

void Textures::setLights()
{
	//** Biztonsag kedveert
	_device->SetRenderState( D3DRS_LIGHTING, TRUE);

	//** Legyen csillanas
//	_device->SetRenderState( D3DRS_SPECULARENABLE, TRUE);

	//** Ambient feny: ahol nem eri direkt feny az objektumokat ott is hat
	_device->SetRenderState( D3DRS_AMBIENT, D3DCOLOR_XRGB(20, 20, 20));

	//---

	//** Irany fenyforras: parhuzamos sugarakban eri a fenye a szinteret
	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(D3DLIGHT9) );

	//** Tipusa
	light.Type    = D3DLIGHT_DIRECTIONAL;
	//** Szine
	light.Diffuse = D3DXCOLOR(0.7f, 0.7f, 0.7f, 1);
	light.Specular = D3DXCOLOR(1, 1, 1, 1);

	//** Iranya
	light.Direction = D3DXVECTOR3(-1, -2, 3);

	//** Adjuk at a device-nak es engedelyezzuk a feny forrast
	_device->SetLight( 0, &light );
	_device->LightEnable( 0, FALSE );

	//---

	//** Pont fenyforras
	D3DLIGHT9 light2;
	ZeroMemory( &light2, sizeof(D3DLIGHT9) );
	light2.Type     = D3DLIGHT_POINT;
	light2.Diffuse  = D3DXCOLOR(0.7f, 0.7f, 0.7f, 1);
	light2.Specular = D3DXCOLOR(1, 1, 1, 1);

	//** Nem iranya van, hanem pozicioja
	light2.Position = D3DXVECTOR3(-0.25f,0.5f,-0.25f);

	light2.Attenuation0 = 0.1f;
	light2.Attenuation1 = 3.0f;
	light2.Range = 2.0f;

	_device->SetLight( 1, &light2 );
	_device->LightEnable( 1, FALSE );

	//---

	//** Spot lampa
	D3DLIGHT9 light3;
	ZeroMemory( &light3, sizeof(D3DLIGHT9) );
	light3.Type    = D3DLIGHT_SPOT;
	light3.Diffuse = D3DXCOLOR(0.7f, 0.7f, 0.7f, 1);
	light3.Specular = D3DXCOLOR(1, 1, 1, 1);

	//** iranya es pozicioja is van
	light3.Position = D3DXVECTOR3(-0.5f,0.5f,-0.5f);
	light3.Direction = D3DXVECTOR3(0.75f, -1, 0.75f);

	//** egyeb parameterek
	light3.Range = 200.0f;
	light3.Theta = 0.5f;
	light3.Phi = 1.0f;
	light3.Falloff = 1.0f;
	light3.Attenuation0 = 0.1f;

	_device->SetLight( 2, &light3 );
	_device->LightEnable( 2, FALSE );

	//---------------------------------------------------------------
	//** Adjunk meg anyagot
	D3DMATERIAL9 material;

	ZeroMemory( &material, sizeof(D3DMATERIAL9) );

	material.Diffuse = D3DXCOLOR(1, 1, 1, 1);
	material.Ambient = D3DXCOLOR(1, 1, 1, 1);

	material.Specular = D3DXCOLOR(1, 1, 1, 1);
	material.Power = 50;

	_device->SetMaterial( &material );
}